!
!     CalculiX - A 3-dimensional finite element program
!              Copyright (C) 1998-2007 Guido Dhondt
!
!     This program is free software; you can redistribute it and/or
!     modify it under the terms of the GNU General Public License as
!     published by the Free Software Foundation(version 2);
!     
!
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of 
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program; if not, write to the Free Software
!     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
      subroutine contactspringforc(xl,konl,vl,imat,elcon,
     &  elas,fn,ncmat_,ntmat_,nope)
!
!     calculates the force of the contact spring: this element
!     consists of a triangular base (independent contact nodes)
!     with an dependent node on top.
!
      implicit none
!
      integer konl(20),i,j,imat,ncmat_,ntmat_,nope,nterms,iflag
!
      real*8 xl(3,20),elas(21),ratio(9),
     &  dc(3),vl(0:3,20),pl(0:3,9),xn(3),al,dd,
     &  alpha,beta,fn(0:3,*),elcon(0:ncmat_,ntmat_,*),
     &  pproj(3),xsj2(3),xs2(3,2),dist,shp2(4,8),xi,et
!
      data iflag /2/
!
!     actual positions of the nodes belonging to the contact spring
!
c      write(*,*) 'start contactspringforc'
c      write(*,*)
      do i=1,nope
         do j=1,3
            pl(j,i)=xl(j,i)+vl(j,i)
         enddo
      enddo
      nterms=nope-1
c      write(*,*) 'nodal positions'
c      write(*,*) (pl(j,1),j=1,3)
c      write(*,*) (pl(j,2),j=1,3)
c      write(*,*) (pl(j,3),j=1,3)
c      write(*,*) (pl(j,4),j=1,3)
c      write(*,*)
!
!     vector dc connects the dependent node with its projection
!     on the independent face
!
      do i=1,3
         pproj(i)=pl(i,nope)
      enddo
      call attach(pl,pproj,nterms,ratio,dist,xi,et)
      do i=1,3
         dc(i)=pl(i,nope)-pproj(i)
      enddo
c      write(*,*) 'pl(1,nope) ',(pl(i,nope),i=1,3)
c      write(*,*) 'dc ',(dc(i),i=1,3)
!
!     determining the jacobian vector on the surface 
!
      if(nterms.eq.8) then
         call shape8q(xi,et,pl,xsj2,xs2,shp2,iflag)
      elseif(nterms.eq.4) then
         call shape4q(xi,et,pl,xsj2,xs2,shp2,iflag)
      elseif(nterms.eq.6) then
         call shape6tri(xi,et,pl,xsj2,xs2,shp2,iflag)
      else
         call shape3tri(xi,et,pl,xsj2,xs2,shp2,iflag)
      endif
!
!     normal on the surface
!
      dd=dsqrt(xsj2(1)*xsj2(1)+xsj2(2)*xsj2(2)+xsj2(3)*xsj2(3))
      do i=1,3
         xn(i)=xsj2(i)/dd
      enddo
c      write(*,*) 'normal'
c      write(*,*) (xn(j),j=1,3)
c      write(*,*)
!
!     distance from surface along normal
!
      al=dc(1)*xn(1)+dc(2)*xn(2)+dc(3)*xn(3)
c      write(*,*) 'al'
c      write(*,*) al
c      write(*,*)
!
!     representative area
!
      if((nterms.eq.8).or.(nterms.eq.4)) then
         dd=dd*4.d0
      else
         dd=dd/2.d0
      endif
!
      alpha=elcon(2,1,imat)*dd
      beta=elcon(1,1,imat)
      if(-beta*al.gt.23.d0-dlog(alpha)) then
         beta=(dlog(alpha)-23.d0)/al
      endif
      elas(1)=alpha*dexp(-beta*al)
      elas(2)=-beta*elas(1)
c      write(*,*) 'alpha,beta,elas(1),elas(2)'
c      write(*,*) alpha,beta,elas(1),elas(2)
c      write(*,*)
c      write(*,*) 'xia,xib,xic'
c      write(*,*) ratio(1),ratio(2),ratio(3)
c      write(*,*)
!
!     forces in the nodes of the contact element
!
      do i=1,3
         do j=1,nterms
            fn(i,konl(j))=fn(i,konl(j))+ratio(j)*elas(1)*xn(i)
         enddo
         fn(i,konl(nope))=fn(i,konl(nope))-elas(1)*xn(i)
      enddo
c      write(*,*) 'springforce= ',-elas(1)*xn(1),-elas(1)*xn(2),
c     &   -elas(1)*xn(3)
c      write(*,*) 'end contactspringforc'
c      write(*,*)
!
      return
      end

