!
!     CalculiX - A 3-dimensional finite element program
!              Copyright (C) 1998-2007 Guido Dhondt
!
!     This program is free software; you can redistribute it and/or
!     modify it under the terms of the GNU General Public License as
!     published by the Free Software Foundation(version 2);
!     
!
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of 
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
!     GNU General Public License for more details.
!
!     You should have received a copy of the GNU General Public License
!     along with this program; if not, write to the Free Software
!     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!
      subroutine contactspringstiff(xl,elas,konl,voldl,s,imat,elcon,
     &  ncmat_,ntmat_,nope)
!
!     calculates the stiffness of the contact spring: this element
!     consists of a triangular base (independent contact nodes)
!     with an dependent node on top.
!
      implicit none
!
      integer konl(20),i,j,imat,ncmat_,ntmat_,k,l,nope,nterms,iflag,
     &  i1
!
      real*8 xl(3,9),elas(21),ratio(9),pproj(3),dist,shp2(4,8),
     &  dc(3),s(60,60),voldl(3,9),pl(0:3,9),xn(3),al,dd,
     &  c1,c2,c3,c4,alpha,beta,elcon(0:ncmat_,ntmat_,*),xsj2(3),
     &  xsju(3,3,8),dxsju(3,8),h(3,8),fpu(3,3,9),xi,et,
     &  xs2(3,2)
!
      data iflag /2/
!
!     actual positions of the nodes belonging to the contact spring
!
c      write(*,*) 'start contactspringstiff'
c      write(*,*)
      do i=1,nope
         do j=1,3
            pl(j,i)=xl(j,i)+voldl(j,i)
         enddo
      enddo
      nterms=nope-1
c      write(*,*) 'nodal positions'
c      write(*,*) (pl(j,1),j=1,3)
c      write(*,*) (pl(j,2),j=1,3)
c      write(*,*) (pl(j,3),j=1,3)
c      write(*,*) (pl(j,4),j=1,3)
c      write(*,*) (pl(j,5),j=1,3)
c      write(*,*) (pl(j,6),j=1,3)
c      write(*,*) (pl(j,7),j=1,3)
c      write(*,*) (pl(j,8),j=1,3)
c      write(*,*) (pl(j,9),j=1,3)
c      write(*,*)
!
!     vector dc connects the dependent node with its projection
!     on the independent face
!
      do i=1,3
         pproj(i)=pl(i,nope)
      enddo
      call attach(pl,pproj,nterms,ratio,dist,xi,et)
      do i=1,3
         dc(i)=pl(i,nope)-pproj(i)
      enddo
!
!     determining the jacobian vector on the surface 
!
      if(nterms.eq.8) then
         call shape8q(xi,et,pl,xsj2,xs2,shp2,iflag)
      elseif(nterms.eq.4) then
         call shape4q(xi,et,pl,xsj2,xs2,shp2,iflag)
      elseif(nterms.eq.6) then
         call shape6tri(xi,et,pl,xsj2,xs2,shp2,iflag)
      else
         call shape3tri(xi,et,pl,xsj2,xs2,shp2,iflag)
      endif
!
!     normal on the surface
!
      dd=dsqrt(xsj2(1)*xsj2(1)+xsj2(2)*xsj2(2)+xsj2(3)*xsj2(3))
      do i=1,3
         xn(i)=xsj2(i)/dd
      enddo
c      write(*,*) 'normal'
c      write(*,*) (xn(j),j=1,3)
c      write(*,*)
!
!     distance from surface along normal
!
      al=dc(1)*xn(1)+dc(2)*xn(2)+dc(3)*xn(3)
c      write(*,*) 'al'
c      write(*,*) al
c      write(*,*)
!
!     alpha and beta, taking the representative area into account
!     (conversion of pressure into force)
!
      if((nterms.eq.8).or.(nterms.eq.4)) then
         alpha=elcon(2,1,imat)*dd*4.d0
      else
         alpha=elcon(2,1,imat)*dd/2.d0
      endif
      beta=elcon(1,1,imat)
      if(-beta*al.gt.23.d0-dlog(alpha)) then
         beta=(dlog(alpha)-23.d0)/al
      endif
      elas(1)=alpha*dexp(-beta*al)
      elas(2)=-beta*elas(1)
c      write(*,*) 'alpha,beta,elas(1),elas(2)'
c      write(*,*) alpha,beta,elas(1),elas(2)
c      write(*,*)
!
      do k=1,nterms
!
!        derivatives of the jacobian vector w.r.t. the displacement
!        vectors
!
         xsju(1,1,k)=0.d0
         xsju(2,2,k)=0.d0
         xsju(3,3,k)=0.d0
         xsju(1,2,k)=shp2(1,k)*xs2(3,2)-shp2(2,k)*xs2(3,1)
         xsju(2,3,k)=shp2(1,k)*xs2(1,2)-shp2(2,k)*xs2(1,1)
         xsju(3,1,k)=shp2(1,k)*xs2(2,2)-shp2(2,k)*xs2(2,1)
         xsju(1,3,k)=-xsju(3,1,k)
         xsju(2,1,k)=-xsju(1,2,k)
         xsju(3,2,k)=-xsju(2,3,k)
!
!        derivatives of the size of the jacobian vector w.r.t. the
!        displacement vectors
!
         do i=1,3
            dxsju(i,k)=xn(1)*xsju(1,i,k)+xn(2)*xsju(2,i,k)+
     &           xn(3)*xsju(3,i,k)
         enddo
!
!        auxiliary variables
!
         do i=1,3
c            h(i,k)=dc(1)*xsju(1,i,k)+dc(2)*xsju(2,i,k)+
c     &             dc(3)*xsju(3,i,k)-ratio(k)*xsj2(i)-al*dxsju(i,k)
            h(i,k)=dc(1)*xsju(1,i,k)+dc(2)*xsju(2,i,k)+
     &             dc(3)*xsju(3,i,k)-al*dxsju(i,k)
         enddo
!
      enddo
c      write(*,*) 'xmua '
c      write(*,*) (xsju(1,i,1),i=1,3)
c      write(*,*) (xsju(2,i,1),i=1,3)
c      write(*,*) (xsju(3,i,1),i=1,3)
c      write(*,*)
c      write(*,*) 'xmub '
c      write(*,*) (xsju(1,i,2),i=1,3)
c      write(*,*) (xsju(2,i,2),i=1,3)
c      write(*,*) (xsju(3,i,2),i=1,3)
c      write(*,*)
c      write(*,*) 'xmuc '
c      write(*,*) (xsju(1,i,3),i=1,3)
c      write(*,*) (xsju(2,i,3),i=1,3)
c      write(*,*) (xsju(3,i,3),i=1,3)
c      write(*,*)
c      write(*,*) 'xmud '
c      write(*,*) (xsju(1,i,4),i=1,3)
c      write(*,*) (xsju(2,i,4),i=1,3)
c      write(*,*) (xsju(3,i,4),i=1,3)
c      write(*,*)
c      write(*,*) 'xmue '
c      write(*,*) (xsju(1,i,5),i=1,3)
c      write(*,*) (xsju(2,i,5),i=1,3)
c      write(*,*) (xsju(3,i,5),i=1,3)
c      write(*,*)
c      write(*,*) 'xmuf '
c      write(*,*) (xsju(1,i,6),i=1,3)
c      write(*,*) (xsju(2,i,6),i=1,3)
c      write(*,*) (xsju(3,i,6),i=1,3)
c      write(*,*)
c      write(*,*) 'xmug '
c      write(*,*) (xsju(1,i,7),i=1,3)
c      write(*,*) (xsju(2,i,7),i=1,3)
c      write(*,*) (xsju(3,i,7),i=1,3)
c      write(*,*)
c      write(*,*) 'xmuh '
c      write(*,*) (xsju(1,i,8),i=1,3)
c      write(*,*) (xsju(2,i,8),i=1,3)
c      write(*,*) (xsju(3,i,8),i=1,3)
c      write(*,*)
c      write(*,*) 'dmua ',(dxsju(i,1),i=1,3)
c      write(*,*)
c      write(*,*) 'dmub ',(dxsju(i,2),i=1,3)
c      write(*,*)
c      write(*,*) 'dmuc ',(dxsju(i,3),i=1,3)
c      write(*,*)
c      write(*,*) 'dmud ',(dxsju(i,4),i=1,3)
c      write(*,*)
c      write(*,*) 'dmue ',(dxsju(i,5),i=1,3)
c      write(*,*)
c      write(*,*) 'dmuf ',(dxsju(i,6),i=1,3)
c      write(*,*)
c      write(*,*) 'dmug ',(dxsju(i,7),i=1,3)
c      write(*,*)
c      write(*,*) 'dmuh ',(dxsju(i,8),i=1,3)
c      write(*,*)
c      write(*,*) 'dc ',(dc(i),i=1,3)
c      write(*,*) 'ha ',(h(i,1),i=1,3)
c      write(*,*) 'hb ',(h(i,2),i=1,3)
c      write(*,*) 'hc ',(h(i,3),i=1,3)
c      write(*,*) 'hd ',(h(i,4),i=1,3)
c      write(*,*) 'he ',(h(i,5),i=1,3)
c      write(*,*) 'hf ',(h(i,6),i=1,3)
c      write(*,*) 'hg ',(h(i,7),i=1,3)
c      write(*,*) 'hh ',(h(i,8),i=1,3)
c      write(*,*)
c      write(*,*) 'xia,xib,xic,xid'
c      write(*,*) ratio(1),ratio(2),ratio(3),ratio(4)
c      write(*,*) 'xie,xif,xig,xih'
c      write(*,*) ratio(5),ratio(6),ratio(7),ratio(8)
c      write(*,*)
!
      c1=1.d0/dd
      c2=c1*c1
      c3=elas(2)*c2
      c4=elas(1)*c1
!
!     derivatives of the forces w.r.t. the displacement vectors
!
      do k=1,nterms
         do i=1,3
            do j=1,3
c               fpu(i,j,k)=-c3*xsj2(i)*h(j,k)
c     &                    +c4*(xn(i)*dxsju(j,k)-xsju(i,j,k))
               fpu(i,j,k)=-c3*xsj2(i)*(h(j,k)-ratio(k)*xsj2(j))
     &                    +c4*(xn(i)*dxsju(j,k)-xsju(i,j,k))
            enddo
         enddo
      enddo
      do i=1,3
         do j=1,3
            fpu(i,j,nope)=-c3*xsj2(i)*xsj2(j)
         enddo
      enddo
c      write(*,*) 'fpua '
c      write(*,*) (fpu(1,i,1),i=1,3)
c      write(*,*) (fpu(2,i,1),i=1,3)
c      write(*,*) (fpu(3,i,1),i=1,3)
c      write(*,*) 'fpub '
c      write(*,*) (fpu(1,i,2),i=1,3)
c      write(*,*) (fpu(2,i,2),i=1,3)
c      write(*,*) (fpu(3,i,2),i=1,3)
c      write(*,*) 'fpuc '
c      write(*,*) (fpu(1,i,3),i=1,3)
c      write(*,*) (fpu(2,i,3),i=1,3)
c      write(*,*) (fpu(3,i,3),i=1,3)
c      write(*,*) 'fpud '
c      write(*,*) (fpu(1,i,4),i=1,3)
c      write(*,*) (fpu(2,i,4),i=1,3)
c      write(*,*) (fpu(3,i,4),i=1,3)
c      write(*,*) 'fpue '
c      write(*,*) (fpu(1,i,5),i=1,3)
c      write(*,*) (fpu(2,i,5),i=1,3)
c      write(*,*) (fpu(3,i,5),i=1,3)
c      write(*,*) 'fpuf '
c      write(*,*) (fpu(1,i,6),i=1,3)
c      write(*,*) (fpu(2,i,6),i=1,3)
c      write(*,*) (fpu(3,i,6),i=1,3)
c      write(*,*) 'fpug '
c      write(*,*) (fpu(1,i,7),i=1,3)
c      write(*,*) (fpu(2,i,7),i=1,3)
c      write(*,*) (fpu(3,i,7),i=1,3)
c      write(*,*) 'fpuh '
c      write(*,*) (fpu(1,i,8),i=1,3)
c      write(*,*) (fpu(2,i,8),i=1,3)
c      write(*,*) (fpu(3,i,8),i=1,3)
c      write(*,*) 'fpui '
c      write(*,*) (fpu(1,i,9),i=1,3)
c      write(*,*) (fpu(2,i,9),i=1,3)
c      write(*,*) (fpu(3,i,9),i=1,3)
c      write(*,*)
!
!     determining the stiffness matrix contributions
!
      do k=1,nterms
         ratio(k)=-ratio(k)
      enddo
      ratio(nope)=1.d0
!
      do k=1,nope
         do l=1,nope
            do i=1,3
               i1=i+(k-1)*3
               do j=1,3
                  s(i1,j+(l-1)*3)=ratio(k)*fpu(i,j,l)
               enddo
            enddo
         enddo
      enddo
c      write(*,*) 'stiffness matrix'
c      do i=1,27
c         write(*,'(15(1x,e11.4))') (s(i,j),j=1,27)
c      enddo
c      write(*,*)
!
c      write(*,*)
c      write(*,*) 'stiffness matrix'
c      do i=1,12
c         write(*,'(12(1x,e11.4))') (s(i,j),j=1,12)
c      enddo
c      write(*,*)
!
!     symmetrizing the matrix
!
      do j=1,3*nope
         do i=1,j-1
            s(i,j)=(s(i,j)+s(j,i))/2.d0
         enddo
      enddo
c      write(*,*) 'end contactspringstiff'
c      write(*,*)
!
      return
      end

