!
!     CalculiX - A 3-dimensional finite element program
!     Copyright (C) 1998-2007 Guido Dhondt
!     
!     This program is free software; you can redistribute it and/or
!     modify it under the terms of the GNU General Public License as
!     published by the Free Software Foundation(version 2);
!     
!     
!     This program is distributed in the hope that it will be useful,
!     but WITHOUT ANY WARRANTY; without even the implied warranty of 
!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
!     GNU General Public License for more details.
!     
!     You should have received a copy of the GNU General Public License
!     along with this program; if not, write to the Free Software
!     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
!     
      subroutine Ts_calc(xflow,Tt,Pt,kappa,r,a,Ts)
!     
!     this subroutine solves the implicit equation
!     f=xflow*dsqrt(Tt)/(a*Pt)-C*(TtdT)**expon*(Ttdt-1)**0.5d0
!     
      implicit none
!
      integer inv
!     
      real*8 xflow,Tt,Pt,Ts,kappa,r,f,df,a,expon,Ts_old,C,TtzTs,
     &     deltaTs,TtzTs_crit, Qred_crit,Qred
!     
      expon=-0.5d0*(kappa+1.d0)/(kappa-1.d0)
!     
      C=dsqrt(2.d0/r*kappa/(kappa-1.d0))
!     
!     f=xflow*dsqrt(Tt)/(a*Pt)-C*(TtdT)**expon*(Ttdt-1)**0.5d0
!
!     df=-C*Ttdt**expon*(expon/Ts*(TtdT-1)**0.5d0
!     &     -0.5d0*TtdT/Ts*(TtdT-1.d0)**(-0.5d0))
!     
      Ts_old=Tt
      Ts=0.99999d0*Tt
!
      if(xflow.lt.0d0) then 
         inv=-1
      else
         inv=1
      endif
      Qred=abs(xflow)*dsqrt(Tt)/(a*Pt)
!     
      TtzTs_crit=(kappa+1.d0)/2.d0
      Qred_crit=C*(TtzTs_crit)**expon*(Ttzts_crit-1.d0)**0.5d0
!     
      if(Qred.ge.Qred_crit) then
!         xflow=inv*Qred_crit/dsqrt(Tt)*A*Pt
         Ts=Tt/TtzTs_crit
!     
         return
!     
      endif
!     
      do 
         Ttzts=Tt/Ts
!     
         f=Qred-C*(TtzTs)**expon*(Ttzts-1.d0)**0.5d0
!     
         df=C*TtzTs**expon*dsqrt(TtzTs-1.d0)
     &        *(-expon/Ts+0.5d0*TtzTs/Ts*(TtzTs-1.d0)**(-1.d0))
!     
         deltaTs=-f/df
!     
         Ts=Ts+deltaTs
!     
         if( (((dabs(Ts-Ts_old)/ts_old).le.1.E-5))
     &        .or.((dabs(Ts-Ts_old)).le.1.E-10)) then
            exit
         endif
         Ts_old=Ts
      enddo
!     
      return
      end
      
      
      
